from ..core import (
    BOB,
    S223,
    Connection,
    ConnectionPoint,
    InletConnectionPoint,
    OutletConnectionPoint,
)
from ..enum import ElectricalPhaseIdentifier, Electricity

_namespace = BOB

# === Generic
# Undefined Electrical


class ElectricalConnection(Connection):
    hasMedium = Electricity
    _class_iri = S223.Conductor

    def electrical_phase(self, phase: ElectricalPhaseIdentifier = None):
        self.hasElectricalPhase = phase
        self._data_graph.add((self._node_iri, S223.hasElectricalPhase, phase._node_iri))


class ElectricalConnectionPoint(ConnectionPoint):
    hasMedium = Electricity

    def electrical_phase(self, phase: ElectricalPhaseIdentifier = None):
        self.hasElectricalPhase = phase
        self._data_graph.add((self._node_iri, S223.hasElectricalPhase, phase._node_iri))


class ElectricalInletConnectionPoint(InletConnectionPoint, ElectricalConnectionPoint):
    _class_iri = S223.InletConnectionPoint


class ElectricalOutletConnectionPoint(OutletConnectionPoint, ElectricalConnectionPoint):
    _class_iri = S223.OutletConnectionPoint


# === AC-10000VLL-1Ph-60Hz
# 1 phase


class Electricity_10000VLL_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC10000VLL_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_10000VLL_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC10000VLL_1Ph_60Hz


class Electricity_10000VLL_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_10000VLL_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_10000VLL_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_10000VLL_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-10000VLL-3Ph-60Hz
# 3 Phases


class Electricity_10000VLL_3Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC10000VLL_3Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_10000VLL_3Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC10000VLL_3Ph_60Hz


class Electricity_10000VLL_3Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_10000VLL_3Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_10000VLL_3Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_10000VLL_3Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-10000VLL-5770VLN-1Ph-60Hz
# 1 phase


class Electricity_10000VLL_5770VLN_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC10000VLL_5770VLN_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_10000VLL_5770VLN_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC10000VLL_5770VLN_1Ph_60Hz


class Electricity_10000VLL_5770VLN_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_10000VLL_5770VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_10000VLL_5770VLN_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint,
    Electricity_10000VLL_5770VLN_1Ph_60HzConnectionPoint,
):
    _class_iri = S223.OutletConnectionPoint


# === AC-10000VLL-5770VLN-3Ph-60Hz
# 3 Phases


class Electricity_10000VLL_5770VLN_3Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC10000VLL_5770VLN_3Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_10000VLL_5770VLN_3Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC10000VLL_5770VLN_3Ph_60Hz


class Electricity_10000VLL_5770VLN_3Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_10000VLL_5770VLN_3Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_10000VLL_5770VLN_3Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint,
    Electricity_10000VLL_5770VLN_3Ph_60HzConnectionPoint,
):
    _class_iri = S223.OutletConnectionPoint


# === AC-110VLN-1Ph-50Hz
# 1 phase


class Electricity_110VLN_1Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC110VLN_1Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_110VLN_1Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC110VLN_1Ph_50Hz


class Electricity_110VLN_1Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_110VLN_1Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_110VLN_1Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_110VLN_1Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


class Electricity_120VLN_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC120VLN_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_120VLN_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC120VLN_1Ph_60Hz


class Electricity_120VLN_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_120VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_120VLN_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_120VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-127VLN-1Ph-50Hz
# 1 phase


class Electricity_127VLN_1Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC127VLN_1Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_127VLN_1Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC127VLN_1Ph_50Hz


class Electricity_127VLN_1Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_127VLN_1Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_127VLN_1Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_127VLN_1Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-139VLN-1Ph-50Hz
# 1 phase


class Electricity_139VLN_1Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC139VLN_1Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_139VLN_1Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC139VLN_1Ph_50Hz


class Electricity_139VLN_1Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_139VLN_1Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_139VLN_1Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_139VLN_1Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-1730VLN-1Ph-60Hz
# 1 phase


class Electricity_1730VLN_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC1730VLN_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_1730VLN_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC1730VLN_1Ph_60Hz


class Electricity_1730VLN_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_1730VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_1730VLN_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_1730VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-1900VLN-1Ph-60Hz
# 1 phase


class Electricity_1900VLN_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC1900VLN_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_1900VLN_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC1900VLN_1Ph_60Hz


class Electricity_1900VLN_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_1900VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_1900VLN_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_1900VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-190VLL-110VLN-1Ph-50Hz
# 1 phase


class Electricity_190VLL_110VLN_1Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC190VLL_110VLN_1Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_190VLL_110VLN_1Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC190VLL_110VLN_1Ph_50Hz


class Electricity_190VLL_110VLN_1Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_190VLL_110VLN_1Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_190VLL_110VLN_1Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_190VLL_110VLN_1Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-190VLL-110VLN-3Ph-50Hz
# 3 Phases


class Electricity_190VLL_110VLN_3Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC190VLL_110VLN_3Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_190VLL_110VLN_3Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC190VLL_110VLN_3Ph_50Hz


class Electricity_190VLL_110VLN_3Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_190VLL_110VLN_3Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_190VLL_110VLN_3Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_190VLL_110VLN_3Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-190VLL-1Ph-50Hz
# 1 phase


class Electricity_190VLL_1Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC190VLL_1Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_190VLL_1Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC190VLL_1Ph_50Hz


class Electricity_190VLL_1Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_190VLL_1Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_190VLL_1Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_190VLL_1Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-190VLL-3Ph-50Hz
# 3 Phases


class Electricity_190VLL_3Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC190VLL_3Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_190VLL_3Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC190VLL_3Ph_50Hz


class Electricity_190VLL_3Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_190VLL_3Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_190VLL_3Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_190VLL_3Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-208VLL-120VLN-1Ph-60Hz
# 1 phase


class Electricity_208VLL_120VLN_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC208VLL_120VLN_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_208VLL_120VLN_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC208VLL_120VLN_1Ph_60Hz


class Electricity_208VLL_120VLN_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_208VLL_120VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_208VLL_120VLN_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_208VLL_120VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-208VLL-120VLN-3Ph-60Hz
# 3 Phases


class Electricity_208VLL_120VLN_3Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC208VLL_120VLN_3Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_208VLL_120VLN_3Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC208VLL_120VLN_3Ph_60Hz


class Electricity_208VLL_120VLN_3Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_208VLL_120VLN_3Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_208VLL_120VLN_3Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_208VLL_120VLN_3Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-208VLL-1Ph-60Hz
# 1 phase


class Electricity_208VLL_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC208VLL_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_208VLL_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC208VLL_1Ph_60Hz


class Electricity_208VLL_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_208VLL_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_208VLL_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_208VLL_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-208VLL-3Ph-60Hz
# 3 Phases


class Electricity_208VLL_3Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC208VLL_3Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_208VLL_3Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC208VLL_3Ph_60Hz


class Electricity_208VLL_3Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_208VLL_3Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_208VLL_3Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_208VLL_3Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-219VLN-1Ph-60Hz
# 1 phase


class Electricity_219VLN_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC219VLN_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_219VLN_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC219VLN_1Ph_60Hz


class Electricity_219VLN_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_219VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_219VLN_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_219VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-220VLL-127VLN-1Ph-50Hz
# 1 phase


class Electricity_220VLL_127VLN_1Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC220VLL_127VLN_1Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_220VLL_127VLN_1Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC220VLL_127VLN_1Ph_50Hz


class Electricity_220VLL_127VLN_1Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_220VLL_127VLN_1Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_220VLL_127VLN_1Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_220VLL_127VLN_1Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-220VLL-127VLN-3Ph-50Hz
# 3 Phases


class Electricity_220VLL_127VLN_3Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC220VLL_127VLN_3Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_220VLL_127VLN_3Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC220VLL_127VLN_3Ph_50Hz


class Electricity_220VLL_127VLN_3Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_220VLL_127VLN_3Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_220VLL_127VLN_3Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_220VLL_127VLN_3Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-220VLL-1Ph-50Hz
# 1 phase


class Electricity_220VLL_1Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC220VLL_1Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_220VLL_1Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC220VLL_1Ph_50Hz


class Electricity_220VLL_1Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_220VLL_1Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_220VLL_1Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_220VLL_1Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-220VLL-3Ph-50Hz
# 3 Phases


class Electricity_220VLL_3Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC220VLL_3Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_220VLL_3Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC220VLL_3Ph_50Hz


class Electricity_220VLL_3Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_220VLL_3Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_220VLL_3Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_220VLL_3Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-231VLN-1Ph-50Hz
# 1 phase


class Electricity_231VLN_1Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC231VLN_1Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_231VLN_1Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC231VLN_1Ph_50Hz


class Electricity_231VLN_1Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_231VLN_1Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_231VLN_1Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_231VLN_1Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-2400VLN-1Ph-60Hz
# 1 phase


class Electricity_2400VLN_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC2400VLN_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_2400VLN_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC2400VLN_1Ph_60Hz


class Electricity_2400VLN_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_2400VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_2400VLN_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_2400VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-240VLL-120VLN-1Ph-60Hz
# 1 phase


class Electricity_240VLL_120VLN_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC240VLL_120VLN_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_240VLL_120VLN_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC240VLL_120VLN_1Ph_60Hz


class Electricity_240VLL_120VLN_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_240VLL_120VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_240VLL_120VLN_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_240VLL_120VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-240VLL-139VLN-1Ph-50Hz
# 1 phase


class Electricity_240VLL_139VLN_1Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC240VLL_139VLN_1Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_240VLL_139VLN_1Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC240VLL_139VLN_1Ph_50Hz


class Electricity_240VLL_139VLN_1Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_240VLL_139VLN_1Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_240VLL_139VLN_1Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_240VLL_139VLN_1Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-240VLL-139VLN-3Ph-50Hz
# 3 Phases


class Electricity_240VLL_139VLN_3Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC240VLL_139VLN_3Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_240VLL_139VLN_3Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC240VLL_139VLN_3Ph_50Hz


class Electricity_240VLL_139VLN_3Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_240VLL_139VLN_3Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_240VLL_139VLN_3Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_240VLL_139VLN_3Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-240VLL-1Ph-50Hz
# 1 phase


class Electricity_240VLL_1Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC240VLL_1Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_240VLL_1Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC240VLL_1Ph_50Hz


class Electricity_240VLL_1Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_240VLL_1Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_240VLL_1Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_240VLL_1Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-240VLL-1Ph-60Hz
# 1 phase


class Electricity_240VLL_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC240VLL_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_240VLL_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC240VLL_1Ph_60Hz


class Electricity_240VLL_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_240VLL_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_240VLL_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_240VLL_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-240VLL-208VLN-120VLN-1Ph-60Hz
# 1 phase


class Electricity_240VLL_208VLN_120VLN_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC240VLL_208VLN_120VLN_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_240VLL_208VLN_120VLN_1Ph_60HzConnectionPoint(
    ElectricalConnectionPoint
):
    hasMedium = Electricity.AC240VLL_208VLN_120VLN_1Ph_60Hz


class Electricity_240VLL_208VLN_120VLN_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint,
    Electricity_240VLL_208VLN_120VLN_1Ph_60HzConnectionPoint,
):
    _class_iri = S223.InletConnectionPoint


class Electricity_240VLL_208VLN_120VLN_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint,
    Electricity_240VLL_208VLN_120VLN_1Ph_60HzConnectionPoint,
):
    _class_iri = S223.OutletConnectionPoint


# === AC-240VLL-208VLN-120VLN-3Ph-60Hz
# 3 Phases


class Electricity_240VLL_208VLN_120VLN_3Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC240VLL_208VLN_120VLN_3Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_240VLL_208VLN_120VLN_3Ph_60HzConnectionPoint(
    ElectricalConnectionPoint
):
    hasMedium = Electricity.AC240VLL_208VLN_120VLN_3Ph_60Hz


class Electricity_240VLL_208VLN_120VLN_3Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint,
    Electricity_240VLL_208VLN_120VLN_3Ph_60HzConnectionPoint,
):
    _class_iri = S223.InletConnectionPoint


class Electricity_240VLL_208VLN_120VLN_3Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint,
    Electricity_240VLL_208VLN_120VLN_3Ph_60HzConnectionPoint,
):
    _class_iri = S223.OutletConnectionPoint


# === AC-240VLL-3Ph-50Hz
# 3 Phases


class Electricity_240VLL_3Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC240VLL_3Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_240VLL_3Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC240VLL_3Ph_50Hz


class Electricity_240VLL_3Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_240VLL_3Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_240VLL_3Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_240VLL_3Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-240VLL-3Ph-60Hz
# 3 Phases


class Electricity_240VLL_3Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC240VLL_3Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_240VLL_3Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC240VLL_3Ph_60Hz


class Electricity_240VLL_3Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_240VLL_3Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_240VLL_3Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_240VLL_3Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-240VLN-1Ph-50Hz
# 1 phase


class Electricity_240VLN_1Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC240VLN_1Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_240VLN_1Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC240VLN_1Ph_50Hz


class Electricity_240VLN_1Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_240VLN_1Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_240VLN_1Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_240VLN_1Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-24VLN-1Ph-50Hz
# 1 phase


class Electricity_24VLN_1Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC24VLN_1Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_24VLN_1Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC24VLN_1Ph_50Hz


class Electricity_24VLN_1Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_24VLN_1Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_24VLN_1Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_24VLN_1Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-24VLN-1Ph-60Hz
# 1 phase


class Electricity_24VLN_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC24VLN_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_24VLN_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC24VLN_1Ph_60Hz


class Electricity_24VLN_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_24VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_24VLN_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_24VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-277VLN-1Ph-60Hz
# 1 phase


class Electricity_277VLN_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC277VLN_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_277VLN_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC277VLN_1Ph_60Hz


class Electricity_277VLN_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_277VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_277VLN_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_277VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-3000VLL-1730VLN-1Ph-60Hz
# 1 phase


class Electricity_3000VLL_1730VLN_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC3000VLL_1730VLN_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_3000VLL_1730VLN_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC3000VLL_1730VLN_1Ph_60Hz


class Electricity_3000VLL_1730VLN_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_3000VLL_1730VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_3000VLL_1730VLN_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_3000VLL_1730VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-3000VLL-1730VLN-3Ph-60Hz
# 3 Phases


class Electricity_3000VLL_1730VLN_3Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC3000VLL_1730VLN_3Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_3000VLL_1730VLN_3Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC3000VLL_1730VLN_3Ph_60Hz


class Electricity_3000VLL_1730VLN_3Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_3000VLL_1730VLN_3Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_3000VLL_1730VLN_3Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_3000VLL_1730VLN_3Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-3000VLL-1Ph-60Hz
# 1 phase


class Electricity_3000VLL_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC3000VLL_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_3000VLL_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC3000VLL_1Ph_60Hz


class Electricity_3000VLL_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_3000VLL_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_3000VLL_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_3000VLL_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-3000VLL-3Ph-60Hz
# 3 Phases


class Electricity_3000VLL_3Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC3000VLL_3Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_3000VLL_3Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC3000VLL_3Ph_60Hz


class Electricity_3000VLL_3Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_3000VLL_3Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_3000VLL_3Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_3000VLL_3Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-3300VLL-1900VLN-1Ph-60Hz
# 1 phase


class Electricity_3300VLL_1900VLN_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC3300VLL_1900VLN_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_3300VLL_1900VLN_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC3300VLL_1900VLN_1Ph_60Hz


class Electricity_3300VLL_1900VLN_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_3300VLL_1900VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_3300VLL_1900VLN_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_3300VLL_1900VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-3300VLL-1900VLN-3Ph-60Hz
# 3 Phases


class Electricity_3300VLL_1900VLN_3Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC3300VLL_1900VLN_3Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_3300VLL_1900VLN_3Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC3300VLL_1900VLN_3Ph_60Hz


class Electricity_3300VLL_1900VLN_3Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_3300VLL_1900VLN_3Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_3300VLL_1900VLN_3Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_3300VLL_1900VLN_3Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-3300VLL-1Ph-60Hz
# 1 phase


class Electricity_3300VLL_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC3300VLL_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_3300VLL_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC3300VLL_1Ph_60Hz


class Electricity_3300VLL_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_3300VLL_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_3300VLL_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_3300VLL_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-3300VLL-3Ph-60Hz
# 3 Phases


class Electricity_3300VLL_3Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC3300VLL_3Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_3300VLL_3Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC3300VLL_3Ph_60Hz


class Electricity_3300VLL_3Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_3300VLL_3Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_3300VLL_3Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_3300VLL_3Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-3460VLN-1Ph-60Hz
# 1 phase


class Electricity_3460VLN_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC3460VLN_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_3460VLN_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC3460VLN_1Ph_60Hz


class Electricity_3460VLN_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_3460VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_3460VLN_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_3460VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-347VLN-1Ph-60Hz
# 1 phase


class Electricity_347VLN_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC347VLN_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_347VLN_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC347VLN_1Ph_60Hz


class Electricity_347VLN_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_347VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_347VLN_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_347VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-380VLL-1Ph-60Hz
# 1 phase


class Electricity_380VLL_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC380VLL_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_380VLL_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC380VLL_1Ph_60Hz


class Electricity_380VLL_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_380VLL_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_380VLL_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_380VLL_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-380VLL-219VLN-1Ph-60Hz
# 1 phase


class Electricity_380VLL_219VLN_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC380VLL_219VLN_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_380VLL_219VLN_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC380VLL_219VLN_1Ph_60Hz


class Electricity_380VLL_219VLN_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_380VLL_219VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_380VLL_219VLN_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_380VLL_219VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-380VLL-219VLN-3Ph-60Hz
# 3 Phases


class Electricity_380VLL_219VLN_3Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC380VLL_219VLN_3Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_380VLL_219VLN_3Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC380VLL_219VLN_3Ph_60Hz


class Electricity_380VLL_219VLN_3Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_380VLL_219VLN_3Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_380VLL_219VLN_3Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_380VLL_219VLN_3Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-380VLL-3Ph-60Hz
# 3 Phases


class Electricity_380VLL_3Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC380VLL_3Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_380VLL_3Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC380VLL_3Ph_60Hz


class Electricity_380VLL_3Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_380VLL_3Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_380VLL_3Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_380VLL_3Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-3810VLN-1Ph-60Hz
# 1 phase


class Electricity_3810VLN_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC3810VLN_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_3810VLN_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC3810VLN_1Ph_60Hz


class Electricity_3810VLN_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_3810VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_3810VLN_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_3810VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-400VLL-1Ph-50Hz
# 1 phase


class Electricity_400VLL_1Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC400VLL_1Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_400VLL_1Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC400VLL_1Ph_50Hz


class Electricity_400VLL_1Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_400VLL_1Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_400VLL_1Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_400VLL_1Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-400VLL-231VLN-1Ph-50Hz
# 1 phase


class Electricity_400VLL_231VLN_1Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC400VLL_231VLN_1Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_400VLL_231VLN_1Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC400VLL_231VLN_1Ph_50Hz


class Electricity_400VLL_231VLN_1Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_400VLL_231VLN_1Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_400VLL_231VLN_1Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_400VLL_231VLN_1Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-400VLL-231VLN-3Ph-50Hz
# 3 Phases


class Electricity_400VLL_231VLN_3Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC400VLL_231VLN_3Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_400VLL_231VLN_3Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC400VLL_231VLN_3Ph_50Hz


class Electricity_400VLL_231VLN_3Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_400VLL_231VLN_3Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_400VLL_231VLN_3Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_400VLL_231VLN_3Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-400VLL-3Ph-50Hz
# 3 Phases


class Electricity_400VLL_3Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC400VLL_3Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_400VLL_3Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC400VLL_3Ph_50Hz


class Electricity_400VLL_3Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_400VLL_3Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_400VLL_3Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_400VLL_3Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-415VLL-1Ph-50Hz
# 1 phase


class Electricity_415VLL_1Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC415VLL_1Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_415VLL_1Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC415VLL_1Ph_50Hz


class Electricity_415VLL_1Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_415VLL_1Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_415VLL_1Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_415VLL_1Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-415VLL-240VLN-1Ph-50Hz
# 1 phase


class Electricity_415VLL_240VLN_1Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC415VLL_240VLN_1Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_415VLL_240VLN_1Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC415VLL_240VLN_1Ph_50Hz


class Electricity_415VLL_240VLN_1Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_415VLL_240VLN_1Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_415VLL_240VLN_1Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_415VLL_240VLN_1Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-415VLL-240VLN-3Ph-50Hz
# 3 Phases


class Electricity_415VLL_240VLN_3Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC415VLL_240VLN_3Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_415VLL_240VLN_3Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC415VLL_240VLN_3Ph_50Hz


class Electricity_415VLL_240VLN_3Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_415VLL_240VLN_3Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_415VLL_240VLN_3Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_415VLL_240VLN_3Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-415VLL-3Ph-50Hz
# 3 Phases


class Electricity_415VLL_3Ph_50HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC415VLL_3Ph_50Hz
    _class_iri = S223.Conductor


class Electricity_415VLL_3Ph_50HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC415VLL_3Ph_50Hz


class Electricity_415VLL_3Ph_50HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_415VLL_3Ph_50HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_415VLL_3Ph_50HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_415VLL_3Ph_50HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-4160VLL-1Ph-60Hz
# 1 phase


class Electricity_4160VLL_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC4160VLL_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_4160VLL_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC4160VLL_1Ph_60Hz


class Electricity_4160VLL_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_4160VLL_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_4160VLL_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_4160VLL_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-4160VLL-2400VLN-1Ph-60Hz
# 1 phase


class Electricity_4160VLL_2400VLN_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC4160VLL_2400VLN_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_4160VLL_2400VLN_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC4160VLL_2400VLN_1Ph_60Hz


class Electricity_4160VLL_2400VLN_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_4160VLL_2400VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_4160VLL_2400VLN_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_4160VLL_2400VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-4160VLL-2400VLN-3Ph-60Hz
# 3 Phases


class Electricity_4160VLL_2400VLN_3Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC4160VLL_2400VLN_3Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_4160VLL_2400VLN_3Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC4160VLL_2400VLN_3Ph_60Hz


class Electricity_4160VLL_2400VLN_3Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_4160VLL_2400VLN_3Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_4160VLL_2400VLN_3Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_4160VLL_2400VLN_3Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-4160VLL-3Ph-60Hz
# 3 Phases


class Electricity_4160VLL_3Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC4160VLL_3Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_4160VLL_3Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC4160VLL_3Ph_60Hz


class Electricity_4160VLL_3Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_4160VLL_3Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_4160VLL_3Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_4160VLL_3Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-480VLL-1Ph-60Hz
# 1 phase


class Electricity_480VLL_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC480VLL_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_480VLL_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC480VLL_1Ph_60Hz


class Electricity_480VLL_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_480VLL_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_480VLL_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_480VLL_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-480VLL-277VLN-1Ph-60Hz
# 1 phase


class Electricity_480VLL_277VLN_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC480VLL_277VLN_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_480VLL_277VLN_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC480VLL_277VLN_1Ph_60Hz


class Electricity_480VLL_277VLN_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_480VLL_277VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_480VLL_277VLN_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_480VLL_277VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-480VLL-277VLN-3Ph-60Hz
# 3 Phases


class Electricity_480VLL_277VLN_3Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC480VLL_277VLN_3Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_480VLL_277VLN_3Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC480VLL_277VLN_3Ph_60Hz


class Electricity_480VLL_277VLN_3Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_480VLL_277VLN_3Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_480VLL_277VLN_3Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_480VLL_277VLN_3Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-480VLL-3Ph-60Hz
# 3 Phases


class Electricity_480VLL_3Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC480VLL_3Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_480VLL_3Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC480VLL_3Ph_60Hz


class Electricity_480VLL_3Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_480VLL_3Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_480VLL_3Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_480VLL_3Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-5770VLN-1Ph-60Hz
# 1 phase


class Electricity_5770VLN_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC5770VLN_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_5770VLN_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC5770VLN_1Ph_60Hz


class Electricity_5770VLN_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_5770VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_5770VLN_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_5770VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-6000VLL-1Ph-60Hz
# 1 phase


class Electricity_6000VLL_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC6000VLL_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_6000VLL_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC6000VLL_1Ph_60Hz


class Electricity_6000VLL_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_6000VLL_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_6000VLL_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_6000VLL_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-6000VLL-3460VLN-1Ph-60Hz
# 1 phase


class Electricity_6000VLL_3460VLN_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC6000VLL_3460VLN_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_6000VLL_3460VLN_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC6000VLL_3460VLN_1Ph_60Hz


class Electricity_6000VLL_3460VLN_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_6000VLL_3460VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_6000VLL_3460VLN_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_6000VLL_3460VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-6000VLL-3460VLN-3Ph-60Hz
# 3 Phases


class Electricity_6000VLL_3460VLN_3Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC6000VLL_3460VLN_3Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_6000VLL_3460VLN_3Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC6000VLL_3460VLN_3Ph_60Hz


class Electricity_6000VLL_3460VLN_3Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_6000VLL_3460VLN_3Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_6000VLL_3460VLN_3Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_6000VLL_3460VLN_3Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-6000VLL-3Ph-60Hz
# 3 Phases


class Electricity_6000VLL_3Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC6000VLL_3Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_6000VLL_3Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC6000VLL_3Ph_60Hz


class Electricity_6000VLL_3Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_6000VLL_3Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_6000VLL_3Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_6000VLL_3Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-600VLL-1Ph-60Hz
# 1 phase


class Electricity_600VLL_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC600VLL_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_600VLL_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC600VLL_1Ph_60Hz


class Electricity_600VLL_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_600VLL_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_600VLL_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_600VLL_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-600VLL-347VLN-1Ph-60Hz
# 1 phase


class Electricity_600VLL_347VLN_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC600VLL_347VLN_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_600VLL_347VLN_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC600VLL_347VLN_1Ph_60Hz


class Electricity_600VLL_347VLN_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_600VLL_347VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_600VLL_347VLN_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_600VLL_347VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-600VLL-347VLN-3Ph-60Hz
# 3 Phases


class Electricity_600VLL_347VLN_3Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC600VLL_347VLN_3Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_600VLL_347VLN_3Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC600VLL_347VLN_3Ph_60Hz


class Electricity_600VLL_347VLN_3Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_600VLL_347VLN_3Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_600VLL_347VLN_3Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_600VLL_347VLN_3Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-600VLL-3Ph-60Hz
# 3 Phases


class Electricity_600VLL_3Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC600VLL_3Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_600VLL_3Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC600VLL_3Ph_60Hz


class Electricity_600VLL_3Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_600VLL_3Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_600VLL_3Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_600VLL_3Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-6600VLL-1Ph-60Hz
# 1 phase


class Electricity_6600VLL_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC6600VLL_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_6600VLL_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC6600VLL_1Ph_60Hz


class Electricity_6600VLL_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_6600VLL_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_6600VLL_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_6600VLL_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-6600VLL-3810VLN-1Ph-60Hz
# 1 phase


class Electricity_6600VLL_3810VLN_1Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC6600VLL_3810VLN_1Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_6600VLL_3810VLN_1Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC6600VLL_3810VLN_1Ph_60Hz


class Electricity_6600VLL_3810VLN_1Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_6600VLL_3810VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_6600VLL_3810VLN_1Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_6600VLL_3810VLN_1Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-6600VLL-3810VLN-3Ph-60Hz
# 3 Phases


class Electricity_6600VLL_3810VLN_3Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC6600VLL_3810VLN_3Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_6600VLL_3810VLN_3Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC6600VLL_3810VLN_3Ph_60Hz


class Electricity_6600VLL_3810VLN_3Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_6600VLL_3810VLN_3Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_6600VLL_3810VLN_3Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_6600VLL_3810VLN_3Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === AC-6600VLL-3Ph-60Hz
# 3 Phases


class Electricity_6600VLL_3Ph_60HzConnection(ElectricalConnection):
    hasMedium = Electricity.AC6600VLL_3Ph_60Hz
    _class_iri = S223.Conductor


class Electricity_6600VLL_3Ph_60HzConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.AC6600VLL_3Ph_60Hz


class Electricity_6600VLL_3Ph_60HzInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_6600VLL_3Ph_60HzConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_6600VLL_3Ph_60HzOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_6600VLL_3Ph_60HzConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === DC-12V
# DC


class Electricity_12VConnection(ElectricalConnection):
    hasMedium = Electricity.DC12V
    _class_iri = S223.Conductor


class Electricity_12VConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.DC12V


class Electricity_12VInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_12VConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_12VOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_12VConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === DC-24V
# DC


class Electricity_24VConnection(ElectricalConnection):
    hasMedium = Electricity.DC24V
    _class_iri = S223.Conductor


class Electricity_24VConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.DC24V


class Electricity_24VInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_24VConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_24VOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_24VConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === DC-380V
# DC


class Electricity_380VConnection(ElectricalConnection):
    hasMedium = Electricity.DC380V
    _class_iri = S223.Conductor


class Electricity_380VConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.DC380V


class Electricity_380VInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_380VConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_380VOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_380VConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === DC-48V
# DC


class Electricity_48VConnection(ElectricalConnection):
    hasMedium = Electricity.DC48V
    _class_iri = S223.Conductor


class Electricity_48VConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.DC48V


class Electricity_48VInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_48VConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_48VOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_48VConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === DC-5V
# DC


class Electricity_5VConnection(ElectricalConnection):
    hasMedium = Electricity.DC5V
    _class_iri = S223.Conductor


class Electricity_5VConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.DC5V


class Electricity_5VInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_5VConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_5VOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_5VConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === DC-6V
# DC


class Electricity_6VConnection(ElectricalConnection):
    hasMedium = Electricity.DC6V
    _class_iri = S223.Conductor


class Electricity_6VConnectionPoint(ElectricalConnectionPoint):
    hasMedium = Electricity.DC6V


class Electricity_6VInletConnectionPoint(
    ElectricalInletConnectionPoint, Electricity_6VConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Electricity_6VOutletConnectionPoint(
    ElectricalOutletConnectionPoint, Electricity_6VConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint



"""
Physical Space subclasses, see also Real Estate Core.
"""

from ..core import BOB, PhysicalSpace
from ..properties.physical import Area

_namespace = BOB


class Site(PhysicalSpace):
    area: Area


class Building(PhysicalSpace):
    area: Area


class Roof(PhysicalSpace):
    area: Area


class Floor(PhysicalSpace):
    area: Area


class Basement(Floor):
    area: Area


class Room(PhysicalSpace):
    area: Area


class Hall(Room):
    area: Area


class Corridor(Room):
    area: Area


class Bathroom(Room):
    area: Area


class Office(Room):
    area: Area


class MechanicalRoom(Room):
    area: Area



from bob.connections.air import AirBidirectionalConnectionPoint
from bob.connections.light import LightVisibleOutletConnectionPoint
from bob.core import BOB, S223, Equipment, PropertyReference

_namespace = BOB


class Door(Equipment):
    _class_iri = S223.Door
    door: AirBidirectionalConnectionPoint
    naturalLight: LightVisibleOutletConnectionPoint

    # Those will come from something else, but be accessible from here.
    openCloseStatus: PropertyReference
    openCloseCommand: PropertyReference



from __future__ import annotations

from typing import Any

from rdflib import URIRef

from ..core import (
    BOB,
    QUANTITYKIND,
    S223,
    UNIT,
    PropertyReference,
    Setpoint,
)
from ..enum import Air
from ..properties import RelativeHumidity
from .sensor import Sensor, split_kwargs

_namespace = BOB


class HumiditySetpoint(Setpoint):
    _class_iri = S223.Setpoint
    hasQuantityKind: URIRef = QUANTITYKIND.RelativeHumidity
    hasUnit: URIRef = UNIT.PERCENT_RH


class HumiditySensor(Sensor):
    _class_iri = S223.HumiditySensor
    observes: PropertyReference  # Temperature

    def __init__(self, **kwargs: Any) -> None:
        _sensor_kwargs, _property_kwargs = split_kwargs(kwargs)

        if "ofMedium" not in _property_kwargs:
            raise ValueError(
                "You must provide ofMedium when defining a humidity sensor"
            )

        super().__init__(**_sensor_kwargs)

        self.observes = RelativeHumidity(
            # isObservedBy=self,
            label=f"{self.label}.Humidity",
            **_property_kwargs,
        )


class AirHumiditySensor(HumiditySensor):
    """
    Air humidity sensor. Can model room sensor or duct sensor.
    """

    _class_iri = S223.HumiditySensor

    def __init__(self, **kwargs: Any) -> None:
        _sensor_kwargs, _property_kwargs = split_kwargs(kwargs)

        super().__init__(ofMedium=Air, **kwargs)



from rdflib import URIRef

from ..core import P223, S223, Node


class NetworkProfile(Node):
    """
    A s223:Controller can be represented by a Network Controller
    (like a BACnet or a Lonworks Equipment).
    This class will be a higher class that will serve to relate
    the s223:Controller to the said bacnet:Device or other.
    """

    _class_iri: URIRef = P223.NetworkProfile



from typing import Any


from bob.properties.states import DaylightDetected

from ..core import (
    BOB,
    S223,
    PropertyReference,
)
from .sensor import Sensor, split_kwargs
from ..enum import Light

_namespace = BOB


class DaylightSensor(Sensor):
    _class_iri = S223.Sensor
    # measuresMedium: Medium = Light
    observes: PropertyReference  # visible light level -- units?

    def __init__(self, **kwargs: Any) -> None:
        _sensor_kwargs, _property_kwargs = split_kwargs(kwargs)

        super().__init__(**_sensor_kwargs)

        self.observes = DaylightDetected(
            # isObservedBy=self,
            label=f"{self.label}.Daylight",
            ofMedium=Light.Visible,
            **_property_kwargs,
        )



from ..core import (
    BOB,
    S223,
    Connection,
    ConnectionPoint,
    InletConnectionPoint,
    InletZoneConnectionPoint,
    Medium,
    OutletConnectionPoint,
    OutletZoneConnectionPoint,
    ZoneConnectionPoint,
)

_namespace = BOB


class OccupancyConnection(Connection):
    hasMedium: Medium
    _class_iri = S223.Connection


class OccupancyConnectionPoint(ConnectionPoint):
    hasMedium: Medium


class OccupancyInletConnectionPoint(OccupancyConnectionPoint, InletConnectionPoint):
    _class_iri = S223.InletConnectionPoint


class OccupancyOutletConnectionPoint(OccupancyConnectionPoint, OutletConnectionPoint):
    _class_iri = S223.OutletConnectionPoint


class OccupancyZoneConnectionPoint(ZoneConnectionPoint):
    hasMedium: Medium
    _class_iri = S223.ZoneConnectionPoint


class OccupancyInletZoneConnectionPoint(
    OccupancyZoneConnectionPoint, InletZoneConnectionPoint
):
    _class_iri = BOB.InletZoneConnectionPoint


class OccupancyOutletZoneConnectionPoint(
    OccupancyZoneConnectionPoint, OutletZoneConnectionPoint
):
    _class_iri = BOB.OutletZoneConnectionPoint



from ..core import (
    BOB,
    S223,
    Connection,
    ConnectionPoint,
    InletConnectionPoint,
    InletZoneConnectionPoint,
    Medium,
    OutletConnectionPoint,
    OutletZoneConnectionPoint,
    ZoneConnectionPoint,
)
from ..enum import Light

_namespace = BOB


# === Light
class LightConnectionPoint(ConnectionPoint):
    hasMedium: Medium = Light


class LightInletConnectionPoint(LightConnectionPoint, InletConnectionPoint):
    pass


class LightOutletConnectionPoint(LightConnectionPoint, OutletConnectionPoint):
    _class_iri = S223.OutletConnectionPoint


class LightZoneConnectionPoint(ZoneConnectionPoint):
    hasMedium: Medium = Light
    _class_iri = BOB.ZoneConnectionPoint


class LightInletZoneConnectionPoint(LightZoneConnectionPoint, InletZoneConnectionPoint):
    _class_iri = BOB.InletZoneConnectionPoint


class LightOutletZoneConnectionPoint(
    LightZoneConnectionPoint, OutletZoneConnectionPoint
):
    _class_iri = BOB.OutletZoneConnectionPoint


class LightVisibleConnection(Connection):
    hasMedium = Light.Visible
    _class_iri = S223.Connection


class LightVisibleConnectionPoint(ConnectionPoint):
    hasMedium = Light.Visible


class LightVisibleInletConnectionPoint(
    LightVisibleConnectionPoint, InletConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class LightVisibleOutletConnectionPoint(
    LightVisibleConnectionPoint, OutletConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


class LightVisibleZoneConnectionPoint(ZoneConnectionPoint):
    hasMedium = Light.Visible
    _class_iri = BOB.ZoneConnectionPoint


class LightVisibleInletZoneConnectionPoint(
    LightVisibleZoneConnectionPoint, InletZoneConnectionPoint
):
    _class_iri = BOB.InletZoneConnectionPoint


class LightVisibleOutletZoneConnectionPoint(
    LightVisibleZoneConnectionPoint, OutletZoneConnectionPoint
):
    _class_iri = BOB.OutletZoneConnectionPoint



from typing import Any


from ..core import (
    BOB,
    S223,
    PropertyReference,
)
from ..enum import Electricity
from ..properties import Amps, OnOffStatus, Volts
from .sensor import Sensor, split_kwargs

_namespace = BOB


class VoltageSensor(Sensor):
    _class_iri = S223.Sensor
    observes: PropertyReference
    hasMinRange: PropertyReference
    hasMaxRange: PropertyReference

    def __init__(self, **kwargs: Any) -> None:
        _sensor_kwargs, _property_kwargs = split_kwargs(kwargs)

        if "ofMedium" not in _property_kwargs:
            # raise ValueError(
            #    "You must provide ofMedium when defining a Voltage Sensor"
            # )
            _property_kwargs["ofMedium"] = Electricity

        super().__init__(**_sensor_kwargs)

        self.observes = Volts(
            # isObservedBy=self,
            label=f"{self.label}.Voltage",
            **_property_kwargs,
        )


class CurrentSensor(Sensor):
    _class_iri = S223.Sensor
    observes: PropertyReference
    hasMinRange: PropertyReference
    hasMaxRange: PropertyReference

    def __init__(self, **kwargs: Any) -> None:
        _sensor_kwargs, _property_kwargs = split_kwargs(kwargs)

        if "ofMedium" not in _property_kwargs:
            _property_kwargs["ofMedium"] = Electricity

        super().__init__(**_sensor_kwargs)

        self.observes = Amps(
            # isObservedBy=self,
            label=f"{self.label}.Amps",
            **_property_kwargs,
        )


class DryContactSensor(Sensor):
    _class_iri = S223.Sensor
    observes: PropertyReference

    def __init__(self, **kwargs: Any) -> None:
        _sensor_kwargs, _property_kwargs = split_kwargs(kwargs)

        if "ofMedium" not in _property_kwargs:
            _property_kwargs["ofMedium"] = Electricity

        super().__init__(**_sensor_kwargs)

        self.observes = OnOffStatus(
            # isObservedBy=self,
            label=f"{self.label}.Amps",
            **_property_kwargs,
        )



from typing import Dict

from bob.properties.states import NormalAlarmStatus

from ...connections.air import AirInletConnectionPoint, AirOutletConnectionPoint
from ...core import P223, Equipment
from ...sensor.particle import (
    CoarseParticulateSensor,
    FineParticulateSensor,
    UltraFineParticulateSensor,
)

_namespace = P223


particlecounter_template = {
    "properties": {"alarmStatus": NormalAlarmStatus},
    "sensors": {
        ("label_of_sensor_1", CoarseParticulateSensor): {
            "hasExternalReference": "bacnet://",
            "comment": "Coarse Particles 10.0um or less",
        },
        ("label_of_sensor_2", FineParticulateSensor): {
            "hasExternalReference": "bacnet://",
            "comment": "Fine Particles 2.5um or less",
        },
        ("label_of_sensor_3", UltraFineParticulateSensor): {
            "hasExternalReference": "bacnet://",
            "comment": "Ultra Fine Particles 1.0um or less",
        },
    },
}


class ParticleCounter(Equipment):
    _class_iri = P223.ParticleCounter
    airInlet: AirInletConnectionPoint
    airOutlet: AirOutletConnectionPoint

    def __init__(self, config: Dict = particlecounter_template, **kwargs):
        config["properties"] = config.get(
            "properties", particlecounter_template["properties"]
        )
        kwargs = {**config.get("params", {}), **kwargs}
        super().__init__(config, **kwargs)



from __future__ import annotations

from typing import Any

from rdflib import URIRef

from ..core import (
    INCLUDE_INVERSE,
    S223,
    LocationReference,
    Node,
    PropertyReference,
    QuantifiableProperty,
    _Sensor,
    logging,
)

# logging
_log = logging.getLogger(__name__)

# namespace
_namespace = S223


def split_kwargs(given_kwargs):
    # specific properties given to a sensor for creation
    # but that must be applied to the observed property, see
    # sorted(QuantifiableObservableProperty._attr_uriref.keys())
    property_attrs = [
        "hasAspect",
        "hasExternalReference",
        "hasQuantityKind",
        "hasSetpoint",
        "hasUnit",
        "hasValue",
        "ofConstituent",
        "ofMedium",
        "ofSubstance",
    ]
    property_kwargs = {}
    sensor_kwargs = {}

    for k, v in given_kwargs.items():
        if v is None:
            continue
        if k in property_attrs:
            property_kwargs[k] = v
        else:
            sensor_kwargs[k] = v

    return (sensor_kwargs, property_kwargs)


def define_sensors(config):
    if not config:
        return []
    sensors = []
    for sensor_label_and_class, sensor_data in config["sensors"].items():
        _label, _cls = sensor_label_and_class
        try:
            if issubclass(_cls, Sensor):
                _cls = _cls
        except:
            raise TypeError("Please provide class for sensor")

        sensors.append(_cls(label=_label, **sensor_data))

    return sensors


class Sensor(_Sensor):
    """
    An equipment meant to observe a property
    """

    _class_iri: URIRef = S223.Sensor
    hasMeasurementPrecision: QuantifiableProperty
    hasMeasurementUncertainty: QuantifiableProperty
    hasMaxRange: QuantifiableProperty
    hasMinRange: QuantifiableProperty
    hasObservationLocation: LocationReference
    observes: PropertyReference

    def __init__(self, **kwargs: Any) -> None:
        _sensor_kwargs, _property_kwargs = split_kwargs(kwargs)
        super().__init__(**_sensor_kwargs)

    @property
    def observedProperty(self):
        """
        When accessing the property, it feels ackward to use sensor.observes
        The terms fit for assignation...but for retrieval, feel unnatural
        So let's try this shortcut
        """
        return self.observes

    def add_hasObservationLocation(self, node: Node) -> None:
        # For now, make that a secret, or we end up with s223.hasObservationLocation
        # self._hasObservationLocation = node

        # link the two together
        self._data_graph.add(
            (self._node_iri, S223.hasObservationLocation, node._node_iri)
        )
        if INCLUDE_INVERSE:
            node.isObservationLocationOf = self

    def __mod__(self, other: Node) -> Node:
        """This sensor measurementLocation taken from some other node."""
        _log.debug(f"Container.__mod__ {self} % {other}")

        self.add_hasObservationLocation(other)
        return self


# class VirtualSensor(Sensor):
#    "Virtal Sensor"
#    _class_iri: URIRef = S223.VirtualSensor
#    # hasObservationLocation: # maxCount = 0
#    hasFunctionInput: Property



from rdflib import URIRef

from ..core import (
    BOB,
    QUANTITYKIND,
    UNIT,
    QuantifiableObservableProperty,
)

_namespace = BOB


class DifferentialTime(QuantifiableObservableProperty):
    hasQuantityKind = QUANTITYKIND.Time


class Hour(DifferentialTime):
    hasUnit: URIRef = UNIT.HR


class Minute(DifferentialTime):
    hasUnit: URIRef = UNIT.MIN


class Second(DifferentialTime):
    hasUnit: URIRef = UNIT.SEC



from .electricity import (
    Amps,
    ElectricApparentEnergy,
    ElectricApparentPower,
    ElectricEnergy,
    ElectricPower,
    ElectricPowerkW,
    ElectricPowerW,
    ElectricReactiveEnergy,
    ElectricReactivePower,
    Frequency,
    PowerFactor,
    Volts,
)
from .flow import Flow
from .force import HP, DifferentialStaticPressure, Nm, Pressure
from .light import Brightness, RelativeLuminousFlux
from .particulate import ParticulateCount
from .rate import AirChangePerHour, Count
from .ratio import RPM, GasConcentration, Percent, PercentCommand, RelativeHumidity
from .setpoint import Setpoint
from .states import (
    DaylightDetected,
    Motion,
    NormalAlarmStatus,
    OccupancyStatus,
    OnOffCommand,
    OnOffStatus,
    Schedule,
    SmokePresence,
)
from .temperature import Temperature
from .volume import Gallons



from ..core import BOB, QUANTITYKIND, UNIT, QuantifiableObservableProperty

_namespace = BOB


class Mbit_per_seconds(QuantifiableObservableProperty):
    hasQuantityKind = QUANTITYKIND.DataRate
    hasUnit = UNIT["MegaBIT-PER-SEC"]


class Kbit_per_seconds(QuantifiableObservableProperty):
    hasQuantityKind = QUANTITYKIND.DataRate
    hasUnit = UNIT["KiloBIT-PER-SEC"]



import logging
from typing import Dict

from rdflib import URIRef

from bob.properties.network import Mbit_per_seconds

from ...connections.electricity import ElectricalInletConnectionPoint
from ...connections.network import (
    EthernetBidirectionalConnectionPoint,
    PoEBidirectionalConnectionPoint,
)
from ...core import (
    BOB,
    S223,
    Equipment,
)
from ...template import template_update

# logging
_log = logging.getLogger(__name__)

# namespace
_namespace = BOB

ethernet_switch_template = {
    "cp": {
        "electricalInlet": ElectricalInletConnectionPoint,
    },
    "properties": {},
}


class EthernetSwitch(Equipment):
    """
    An Ethernet Switch
    """

    _class_iri: URIRef = S223.EthernetSwitch

    def __init__(self, config: Dict = None, **kwargs):
        if "ports" in kwargs:
            _number_of_ports = int(kwargs.pop("ports"))
        else:
            raise ValueError("Please provide number of IP ports using ports=x")
        if "data_rate" in kwargs:
            _data_rate = float(kwargs.pop("data_rate"))
        else:
            raise ValueError("Please provide data rate using data_rate=x in Mbit/s")
        _config = template_update(ethernet_switch_template, config)
        for i, each in enumerate(range(_number_of_ports)):
            _config["cp"][f"port{i}"] = EthernetBidirectionalConnectionPoint
        kwargs = {**_config.get("params", {}), **kwargs}
        super().__init__(_config, **kwargs)
        for k, v in self._connection_points.items():
            if isinstance(v, EthernetBidirectionalConnectionPoint):
                v.data_rate = Mbit_per_seconds(_data_rate)


class PoESwitch(Equipment):
    """
    An Ethernet Switch
    """

    _class_iri: URIRef = S223.PoESwitch

    def __init__(self, config: Dict = None, **kwargs):
        if "ports" in kwargs:
            _number_of_ports = int(kwargs.pop("ports"))
        else:
            raise ValueError("Please provide number of IP ports using ports=x")
        if "data_rate" in kwargs:
            _data_rate = float(kwargs.pop("data_rate"))
        else:
            raise ValueError("Please provide data rate using data_rate=x in Mbit/s")
        _config = template_update(ethernet_switch_template, config)
        for i, each in enumerate(range(_number_of_ports)):
            _config["cp"][f"port{i}"] = PoEBidirectionalConnectionPoint
        kwargs = {**_config.get("params", {}), **kwargs}
        super().__init__(_config, **kwargs)
        for k, v in self._connection_points.items():
            if isinstance(v, EthernetBidirectionalConnectionPoint):
                v.data_rate = Mbit_per_seconds(_data_rate)



from typing import Dict

from ...producer.causality import Causality
from ...properties.electricity import Amps
from ...properties.ratio import PercentCommand
from ...properties.states import OnOffCommand, OnOffStatus

from ...enum import Electricity
from ...connections import electricity as elec_cnx
from ...connections.controlsignal import OnOffSignalOutletConnectionPoint
from ...core import (
    BOB,
    P223,
    Equipment,
)
from ...properties.time import Hour
from ...sensor.electricity import CurrentSensor
from ...template import template_update

_namespace = BOB

# TODO : Use templates

switch_template = {
    "cp": {
        "electricalInlet": elec_cnx.Electricity_120VLN_1Ph_60HzInletConnectionPoint,
        "electricalOutlet": elec_cnx.Electricity_120VLN_1Ph_60HzOutletConnectionPoint,
    },
    "properties": {
        ("amps", Amps): {},
    },
}


class Switch(Equipment):
    _class_iri = P223.ElectricalSwitch
    # electricalInlet: ElectricalInletConnectionPoint
    # electricalOutlet: ElectricalOutletConnectionPoint
    hasMaxRange: Amps
    onOffStatus: OnOffStatus
    onOffCommand: OnOffCommand

    def __init__(self, config: Dict = None, **kwargs):
        _config = template_update(switch_template, config)
        kwargs = {**_config.get("params", {}), **kwargs}

        super().__init__(_config, **kwargs)


class SinglePoleSwitch(Switch):
    """
    One inlet and one outlet
    hasMaxRange = current max of switch
    A rule could check inlet and outlet are same class
    """

    _cross_ref = {
        "120": (
            elec_cnx.Electricity_120VLN_1Ph_60HzInletConnectionPoint,
            elec_cnx.Electricity_120VLN_1Ph_60HzOutletConnectionPoint,
        ),
        "277": (
            elec_cnx.Electricity_277VLN_1Ph_60HzInletConnectionPoint,
            elec_cnx.Electricity_277VLN_1Ph_60HzOutletConnectionPoint,
        ),
        "347": (
            elec_cnx.Electricity_347VLN_1Ph_60HzInletConnectionPoint,
            elec_cnx.Electricity_347VLN_1Ph_60HzOutletConnectionPoint,
        ),
    }

    def __init__(self, config: Dict = None, **kwargs):
        _config = template_update(switch_template, config)
        if config:
            _config.update(config)
        _voltage = kwargs.pop("voltage") if "voltage" in kwargs else None
        if _voltage:
            _electricalInlet, _electricalOutlet = self._cross_ref[str(_voltage)]
            _config["cp"]["electricalInlet"] = _electricalInlet
            _config["cp"]["electricalOutlet"] = _electricalOutlet
        kwargs = {**_config.get("params", {}), **kwargs}

        super().__init__(_config, **kwargs)


current_relay_template = {
    "cp": {"dryContact": OnOffSignalOutletConnectionPoint},
    "properties": {("amps_rating", Amps): {}, ("onOffStatus", OnOffStatus): {}},
}


class CurrentRelay(Equipment):
    """
    Current detection Equipment that gives a OnOff status by the action
    of a dry contact when electricity is detected.

    This serves as motor status sensor

    """

    _class_iri = P223.CurrentRelay

    def __init__(self, config: Dict = None, **kwargs):
        _config = template_update(current_relay_template, config)
        kwargs = {**_config.pop("params", {}), **kwargs}

        # _hasObservationLocation = kwargs.pop("hasObservationLocation", None)

        _label = kwargs["label"]

        super().__init__(_config, **kwargs)

        sensor = CurrentSensor(
            label="currentSensor",
            ofMedium=Electricity,
        )
        # sensor % _hasObservationLocation
        self > sensor
        status_producer = Causality(label="statusProducer")
        status_producer.cause_input << sensor.observedProperty
        status_producer.effect_output >> self["onOffStatus"]
        self > status_producer
        # self["onOffStatus"] += RunStatus
        # self.dryContact += RunStatus


class TimerSwitch(SinglePoleSwitch):
    """
    Manuel switch with integrated timer
    Typically used for exhaust fan in bathrooms for example
    """

    delay: Hour
    onOffStatus: OnOffStatus
    onOffCommand: OnOffCommand

    def __init__(self, config: Dict = None, **kwargs):
        _config = template_update(current_relay_template, config)
        kwargs = {**_config.pop("params", {}), **kwargs}
        _delay = kwargs.pop("delay") if "delay" in kwargs else None

        if _delay:
            _config["properties"][("delay", Hour)] = {"hasValue": _delay}
        super().__init__(_config, **kwargs)


class DimmableSwitch(SinglePoleSwitch):
    """
    Manuel dimmable switch

    """

    def __init__(self, config: Dict = None, **kwargs):
        _config = template_update(current_relay_template, config)
        kwargs = {**_config.pop("params", {}), **kwargs}
        _dimmer_command = (
            kwargs.pop("dimmer_command") if "dimmer_command" in kwargs else 0
        )
        _config["properties"][("dimmer_command", PercentCommand)] = {
            "hasValue": _dimmer_command
        }
        super().__init__(_config, **kwargs)



from ..core import BOB, PropertyReference
from ..producer import Function
from ..properties import PercentCommand, Temperature

_namespace = BOB

# WIP : For now, it won't be in hvacspace by default


class IndoorAir(Function):
    # uses_input
    heating: PercentCommand
    cooling: PercentCommand
    # produces_output
    temperture: Temperature
    hasOccupancySensor: PropertyReference
    # What if I need to connect more than 1 ???
    # def __init__(self, **kwargs):



import logging
from typing import Dict

from ...connections.air import AirInletConnectionPoint
from ...core import BOB, S223, BoundaryConnectionPoint, Equipment, System
from ... import application
from ...equipment.hvac.damper import Damper
from ...template import template_update

# logging
_log = logging.getLogger(__name__)

# namespace
_namespace = BOB

TerminalUnit_template = {
    "equipment": {
        ("damper", Damper): {"comment": "VAV Box Damper including"},
        # ("flow_sensor", AirFlowSensor): {"comment": "Air flow sensor"}
    },
}


class SingleDuctTerminal(System, application.SingleDuctTerminal):
    # _class_iri = S223.SingleDuctTerminal
    airInlet: BoundaryConnectionPoint
    airOutlet: BoundaryConnectionPoint

    def __init__(self, config: Dict = None, **kwargs):
        _config = template_update(TerminalUnit_template, config=config)
        kwargs = {**_config.pop("params", {}), **kwargs}
        _log.info(f"SingleDuctTerminal.__init__ {_config} {kwargs}")
        super().__init__(_config, **kwargs)


# Generic
class GenericSingleDuctTerminal(Equipment, application.SingleDuctTerminal):
    _class_iri = S223.SingleDuctTerminal
    airInlet: AirInletConnectionPoint
    airOutlet: AirInletConnectionPoint

    def __init__(self, config: Dict = None, **kwargs):
        _config = template_update({}, config=config)
        kwargs = {**_config.pop("params", {}), **kwargs}
        _log.info(f"GenericSingleDuctTerminal.__init__ {_config} {kwargs}")
        super().__init__(_config, **kwargs)
        self.airOutlet.paired_to(self.airInlet)



from typing import Dict


from ...connections.air import (
    AirInletConnectionPoint,
    AirOutletConnectionPoint,
)
from ...core import BOB, S223, Equipment, logging
from ...template import template_update, configure_relations

# logging
_log = logging.getLogger(__name__)

# namespace
_namespace = BOB


class Damper(Equipment):
    _class_iri = S223.Damper
    airInlet: AirInletConnectionPoint
    airOutlet: AirOutletConnectionPoint

    def __init__(self, config: Dict = None, **kwargs):
        _config = template_update({}, config=config)
        kwargs = {**_config.pop("params", {}), **kwargs}
        _log.info(f"Damper.__init__ {_config} {kwargs}")
        _relations = _config.pop("relations", [])
        super().__init__(_config, **kwargs)
        configure_relations(self, _relations)
        self.airOutlet.paired_to(self.airInlet)



from __future__ import annotations

from typing import Any

from ..core import (
    BOB,
    S223,
    PropertyReference,
)
from ..enum import Air, Particulate
from ..properties import ParticulateCount
from .sensor import Sensor, split_kwargs

_namespace = BOB


class ParticulateSensor(Sensor):
    _class_iri = S223.ParticulateSensor
    observes: PropertyReference  # ParticulateCount

    def __init__(self, **kwargs: Any) -> None:
        _sensor_kwargs, _property_kwargs = split_kwargs(kwargs)

        super().__init__(**_sensor_kwargs)

        self.observes = ParticulateCount(
            label=f"{self.label}.ParticulateCount",  # needs more focus
            ofMedium=Air,
            **_property_kwargs,
        )


class UltraFineParticulateSensor(ParticulateSensor):
    _class_iri = S223.ParticulateSensor
    "PM 1.0 Count"
    comment = "Ultra Fine Particulate Sensor"

    # measuresSubstance: Substance = PM1_0
    def __init__(self, **kwargs):
        super().__init__(ofSubstance=Particulate.PM1_0, **kwargs)


class FineParticulateSensor(ParticulateSensor):
    _class_iri = S223.ParticulateSensor
    "PM 2.5 Count"
    comment = "Fine Particulate Sensor"

    # measuresSubstance: Substance = PM2_5
    def __init__(self, **kwargs):
        super().__init__(ofSubstance=Particulate.PM2_5, **kwargs)


class CoarseParticulateSensor(ParticulateSensor):
    _class_iri = S223.ParticulateSensor
    "PM 10 Count"
    comment = "Coarse Particulate Sensor"

    # measuresSubstance: Substance = PM10_0
    def __init__(self, **kwargs):
        super().__init__(ofSubstance=Particulate.PM10_0, **kwargs)



from rdflib import XSD, Literal

from ..core import (
    BOB,
    QUANTITYKIND,
    QUDT,
    UNIT,
    Medium,
    QuantifiableObservableProperty,
)

_namespace = BOB

# all = [HP, Pressure, DifferentialStaticPressure]


class HP(QuantifiableObservableProperty):
    hasQuantityKind = QUANTITYKIND.Power
    hasUnit = UNIT.HP


class Nm(QuantifiableObservableProperty):
    hasQuantityKind = QUANTITYKIND.Torque
    hasUnit = UNIT["N-M"]


class Pressure(QuantifiableObservableProperty):
    hasQuantityKind = QUANTITYKIND.Pressure


class DifferentialStaticPressure(QuantifiableObservableProperty):
    hasQuantityKind = QUANTITYKIND.ForcePerArea
    ofMedium: Medium  # set from the sensor

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self._data_graph.add(
            (self._node_iri, QUDT.isDeltaQuantity, Literal(True, datatype=XSD.boolean))
        )



import logging
from typing import Dict

from rdflib import URIRef

from ...connections.air import (
    AirBidirectionalConnectionPoint,
    AirInletConnectionPoint,
    AirOutletConnectionPoint,
)
from ...core import BOB, S223, Equipment
from ...template import configure_relations, template_update  # logging

_namespace = BOB
_log = logging.getLogger(__name__)

"""
chilledWaterCoil_template = {
    "params": {"label": "Name", "comment": "Description"},
    "sensors": {},
    "equipment": {("valve", Equipment): {"comment": "SubDev comment"}},
}
"""

# SEMANTIC QUESTION
# here, that could be a good way to define the coil and its valve...
# but the valve connect to the coil
# can this be considered "contained" in the Coil Equipment ?
# Should this b ea system


class AirHeatExchanger(Equipment):
    _class_iri: URIRef = S223.AirHeatExchanger
    supplyAirInlet: AirInletConnectionPoint
    supplyAirOutlet: AirOutletConnectionPoint
    exhaustAirInlet: AirInletConnectionPoint
    exhaustAirOutlet: AirOutletConnectionPoint

    def __init__(self, config: Dict = None, **kwargs):
        _config = template_update({}, config=config)
        kwargs = {**_config.pop("params", {}), **kwargs}
        _log.info(f"AirHeatExchanger.__init__ {_config} {kwargs}")
        _relations = _config.pop("relations", [])
        super().__init__(_config, **kwargs)
        configure_relations(self, _relations)
        self.supplyAirOutlet.paired_to(self.supplyAirInlet)
        self.exhaustAirOutlet.paired_to(self.exhaustAirInlet)


class Accumulator(Equipment):
    """
    Type of heat exchanger that accumulate heat from exhaust air
    passing through it. Then air flow will switch side. Air will flow
    so outdoor air will use energy accumulated in the substrat. This cycle
    will repeat.
    """

    _class_iri: URIRef = S223.Equipment
    outdoorSide: AirBidirectionalConnectionPoint
    indoorSide: AirBidirectionalConnectionPoint


class Accumulator4SidesDuct(Equipment):
    """
    This is part of the exchanger and allow air to comes in or out of accumulator
    depending on the position of the pneumatic damper
    Return and supply are directional, but accumulator 1 and 2 are bidirectional
    """

    _class_iri: URIRef = S223.Equipment
    accumulator1Connection: AirBidirectionalConnectionPoint
    accumulator2Connection: AirBidirectionalConnectionPoint
    supplyDuctOutlet: AirOutletConnectionPoint
    returnDuctInlet: AirInletConnectionPoint



from ..core import P223, ExternalReference

_namespace = P223


class NiagaraORDReference(ExternalReference):
    _class_iri = None


class TimeSeriesReference(ExternalReference):
    _class_iri = None



# Controller
from ...connections.controlsignal import (
    ModulationSignalInletConnectionPoint as AnalogInput,
)
from ...connections.controlsignal import (
    ModulationSignalOutletConnectionPoint as AnalogOutput,
)
from ...connections.controlsignal import OnOffSignalInletConnectionPoint as BinaryInput
from ...connections.controlsignal import (
    OnOffSignalOutletConnectionPoint as BinaryOutput,
)



"""
Producers

This is an adaptation of the S223 Function Blocks
"""

from __future__ import annotations

import inspect
import logging
from typing import Any, AnyStr, Dict

from rdflib import Literal, URIRef  # type: ignore

from ..core import (
    BOB,
    G36,
    INCLUDE_INVERSE,
    P223,
    S223,
    Container,
    Equipment,
    LocationReference,
    Node,
    Property,
    _Producer,
    data_graph,
)
from ..equipment.control import AnalogInput, AnalogOutput, BinaryInput, BinaryOutput
from ..functions import Function, FunctionInput, FunctionOutput
from ..multimethods import multimethod
from ..template import template_update

# logging
_log = logging.getLogger(__name__)

# namespace
_namespace = BOB


class old_Producer(Container, Node):
    "Placeholder to prevent circular reference"
    _class_iri: URIRef = P223.Producer

    def __init__(self, config: Dict[str, Any] = {}, *args, **kwargs: Any) -> None:
        _log.debug(f"Producer.__init__ {config} {args} {kwargs}")

        # if there are "params" in the configuation, use those as defaults for
        # kwargs and allow them to be overriden by additional kwargs
        # if config and "params" in config:
        #     kwargs = {**config["params"], **kwargs}

        # When passing kwargs to create an instance of a class, some datatype
        # are not yet visible in the chain of creation. This lead to
        # ex. TypeError: unexpected keyword argument: waterInlet
        # By removing properties and connection points from kwargs and explicitly
        # putting them in config, it should be better
        _config = dict(config.items())
        for attr_name, attr_value in kwargs.copy().items():
            if inspect.isclass(attr_value):
                # if issubclass(attr_value, Property):
                #    config["properties"] = (
                #        {**config["properties"], **{attr_name: kwargs.pop(attr_name)}}
                #        if "properties" in config.keys()
                #        else {attr_name: kwargs.pop(attr_name)}
                #    )
                if issubclass(attr_value, ConnectionPoint):
                    # Beware here... _Producer are function Block so...FB in and out only....
                    _config["cp"] = (
                        {**_config["cp"], **{attr_name: kwargs.pop(attr_name)}}
                        if "cp" in _config.keys()
                        else {attr_name: kwargs.pop(attr_name)}
                    )

        super().__init__(*args, **kwargs)

        if _config:
            for group_name, group_items in _config.items():
                if group_name == "params":
                    continue
                if group_name == "cp":
                    for thing_name, thing_class in group_items.items():
                        setattr(
                            self,
                            thing_name,
                            thing_class(self, label=f"{self.label}.{thing_name}"),
                        )
                    continue
                things = []
                for (thing_name, thing_class), thing_kwargs in group_items.items():
                    if thing_name in self:
                        raise ValueError(f"label already used: {self[thing_name]}")
                    thing = thing_class(label=thing_name, **thing_kwargs)

                    if isinstance(thing, (_Producer)):
                        self > thing
                    if isinstance(thing, Property):
                        self[thing_name] = thing
                        self.add_property(thing)

                    things.append(thing)

                setattr(self, "_" + group_name, things)


@multimethod
def contains_mm(producer: Producer, sub_producer: _Producer) -> None:
    """Producer > Producer"""
    _log.info(f"producer {producer} contains producer {sub_producer}")

    producer._data_graph.add((producer._node_iri, BOB.contains, sub_producer._node_iri))


#
#   Function Inputs and Outputs
#


class ProducerInput(Node):
    _class_iri: URIRef = P223.ProducerInput
    hasCauseLocation: LocationReference

    def __init__(self, function_block: Producer, **kwargs: Any) -> None:
        _log.debug(
            f"ProducerInput({self.__class__.__name__}).__init__ {function_block} {kwargs}"
        )

        super().__init__(**kwargs)

        data_graph.add((function_block._node_iri, S223.hasInput, self._node_iri))

    def __rshift__(self, other: Any) -> Any:
        """Build a connection from this thing to another thing."""
        connect_mm(self, other)
        return other

    def __lshift__(self, other: Any) -> Any:
        """Build a connection to this thing from another thing."""
        connect_mm(other, self)
        return self


class ProducerOutput(Node):
    _class_iri: URIRef = P223.ProducerOutput
    hasEffectLocation: LocationReference

    def __init__(self, function_block: Producer, **kwargs: Any) -> None:
        _log.debug(
            f"ProducerOutput({self.__class__.__name__}).__init__ {function_block} {kwargs}"
        )

        super().__init__(**kwargs)

        data_graph.add((function_block._node_iri, S223.hasOutput, self._node_iri))

    def add_hasEffectLocation(self, node: Node) -> None:
        # Must be from a sensor
        self.hasEffectLocation = node

        # link the two together
        self._data_graph.add((self._node_iri, P223.hasEffectLocation, node._node_iri))
        if INCLUDE_INVERSE:
            node.isEffectLocationOf = self

    def __rshift__(self, other: Any) -> Any:
        """Build a connection from this thing to another thing."""
        connect_mm(self, other)
        return other

    def __lshift__(self, other: Any) -> Any:
        """Build a connection to this thing from another thing."""
        connect_mm(other, self)
        return self

    def __mod__(self, other: Node) -> Node:
        """This producer output has effect location on other node."""
        _log.debug(f"Container.__mod__ {self} % {other}")

        self.add_hasEffectLocation(other)
        return self


@multimethod
def connect_mm(
    output_connector: ProducerOutput, input_connector: ProducerInput
) -> None:
    """ProducerOutput >> ProducerInput"""
    _log.info(f"connect from {output_connector} to {input_connector}")

    data_graph.add(
        (output_connector._node_iri, S223.connect, input_connector._node_iri)
    )


@multimethod
def connect_mm(prop: Property, input_connector: ProducerInput) -> None:
    """Property >> ProducerInput"""
    _log.info(f"connect from {prop} to {input_connector}")

    data_graph.add((input_connector._node_iri, S223.uses, prop._node_iri))


@multimethod
def connect_mm(output_connector: ProducerOutput, prop: Property) -> None:
    """ProducerOutput >> Property"""
    _log.info(f"connect from {output_connector} to {prop}")

    data_graph.add((output_connector._node_iri, S223.produces, prop._node_iri))


@multimethod
def connect_mm(output_connector: ProducerOutput, cp: AnalogOutput) -> None:
    """ProducerOutput >> Property"""
    _log.info(f"connect from {output_connector} to {cp}")

    data_graph.add((cp._node_iri, P223.hasProducerOutput, output_connector._node_iri))


@multimethod
def connect_mm(output_connector: ProducerOutput, cp: BinaryOutput) -> None:
    """ProducerOutput >> Controller connection point"""
    _log.info(f"connect from {output_connector} to {cp}")

    data_graph.add((cp._node_iri, P223.hasProducerOutput, output_connector._node_iri))


@multimethod
def connect_mm(output_connector: ProducerOutput, cp: AnalogOutput) -> None:
    """ProducerOutput >> Controller connection point"""
    _log.info(f"connect from {output_connector} to {cp}")

    data_graph.add((cp._node_iri, P223.hasProducerOutput, output_connector._node_iri))


#
#   Connector types, parameters, and constants
#


class G36AnalogInput(FunctionInput):
    _class_iri: URIRef = G36.AnalogInput


class G36AnalogOutput(FunctionOutput):
    _class_iri: URIRef = G36.AnalogOutput


class G36DigitalInput(FunctionInput):
    _class_iri: URIRef = G36.DigitalInput


class G36DigitalOutput(FunctionOutput):
    _class_iri: URIRef = G36.DigitalOutput


#
#   Producer
#


class Producer(_Producer):
    """
    Producers are black boxes representing causality.
    Their inputs are causes and they produce an effect on a property
    It is very similar to a function, but it is meant to show the relation
    between an input and an output of something not-driven by an algorithm.
    A common example would be a relay. You feed it voltage and the relay contact
    is actuated.
    """

    _class_iri: URIRef = P223.Producer

    def __init__(self, config: Dict = None, **kwargs):
        _config = template_update({}, config=config)
        kwargs = {**_config.pop("params", {}), **kwargs}

        _log.debug(f"Producer.__init__ {kwargs}")

        # resolve annotations if necessary
        if not self._resolved:
            self._resolve_annotations()
        _log.debug("    - continue Producer.__init__")

        # pull out the inputs and outputs
        connector_inits: Dict[str, Any] = {}
        for attr_name, attr_type in self._nodes.items():
            if inspect.isclass(attr_type) and (attr_name in kwargs):
                if issubclass(attr_type, (ProducerInput, ProducerOutput)):
                    connector_inits[attr_name] = kwargs.pop(attr_name)
        _log.debug(f"    - connector_inits: {connector_inits}")
        _log.debug(f"    - remaining kwargs: {kwargs}")

        # continue with initialization
        # super().__init__(_config, **kwargs)
        super().__init__(**kwargs)

        # instantiate and associate all of the connectors
        self._connectors = {}
        for attr_name, attr_type in self._nodes.items():
            if not inspect.isclass(attr_type):
                continue

            if issubclass(attr_type, (ProducerInput, ProducerOutput)):
                # build an instance of this connector
                attr_element = attr_type(self, label=self.label + "." + attr_name)
                self._connectors[attr_name] = attr_element
                _log.debug(f"    - connector {attr_name}: {attr_element}")

                if attr_name in connector_inits:
                    _log.debug(f"        - init: {connector_inits[attr_name]}")
                    if issubclass(attr_type, ProducerInput):
                        connector_inits[attr_name] >> attr_element
                    if issubclass(attr_type, ProducerOutput):
                        attr_element >> connector_inits[attr_name]

                setattr(self, attr_name, attr_element)

        if _config:
            for group_name, group_items in _config.items():
                if group_name == "params":
                    continue
                if group_name == "cp":
                    for thing_name, thing_class in group_items.items():
                        setattr(
                            self,
                            thing_name,
                            thing_class(self, label=f"{self.label}.{thing_name}"),
                        )
                    continue
                things = []
                for (thing_name, thing_class), thing_kwargs in group_items.items():
                    if thing_name in self:
                        raise ValueError(f"label already used: {self[thing_name]}")
                    thing = thing_class(label=thing_name, **thing_kwargs)

                    if isinstance(thing, Producer):
                        self > thing
                    if isinstance(thing, Property):
                        self[thing_name] = thing
                        self.add_property(thing)

                    things.append(thing)

                setattr(self, "_" + group_name, things)

    def uses(
        self,
        prop: Property,
        klass: ProducerInput = ProducerInput,
        label: AnyStr = "input",
    ) -> None:
        connector = klass(self, label=f"{self.label}.{label}")
        setattr(self, label, connector)
        prop >> connector

    def produces(
        self,
        prop: Property,
        klass: ProducerOutput = ProducerOutput,
        label: AnyStr = "output",
    ) -> None:
        connector = klass(self, label=f"{self.label}.{label}")
        setattr(self, label, connector)
        connector >> prop


@multimethod
def contains_mm(producer: Producer, sub_producer: Producer) -> None:
    """Producer > Producer"""
    _log.info(f"producer {producer} contains producer {sub_producer}")

    producer._data_graph.add((producer._node_iri, BOB.contains, sub_producer._node_iri))


@multimethod
def contains_mm(equipment: Equipment, producer: Producer) -> None:
    """Equipment > Producer"""
    _log.info(f"equipment {equipment} contains producer {producer}")

    producer._data_graph.add((equipment._node_iri, BOB.contains, producer._node_iri))



from ..core import (
    BOB,
    S223,
    BidirectionalConnectionPoint,
    Connection,
    ConnectionPoint,
    InletConnectionPoint,
    Medium,
    OutletConnectionPoint,
)
from ..enum import Fluid, GlycolSolution_15Percent, GlycolSolution_30Percent, Water

_namespace = BOB


# Connections


# === Generic Fluid
class FluidConnection(Connection):
    _volatile = ("hasMedium",)
    hasMedium: Fluid = Water
    _class_iri = S223.Connection


class FluidConnectionPoint(ConnectionPoint):
    _volatile = ("hasMedium",)
    hasMedium: Fluid = Water


class FluidInletConnectionPoint(FluidConnectionPoint, InletConnectionPoint):
    _class_iri = S223.InletConnectionPoint


class FluidOutletConnectionPoint(FluidConnectionPoint, OutletConnectionPoint):
    _class_iri = S223.OutletConnectionPoint


class FluidBidirectionalConnectionPoint(
    FluidConnectionPoint, BidirectionalConnectionPoint
):
    _class_iri = S223.BidirectionalConnectionPoint


# === WATER


class WaterConnection(Connection):
    _volatile = ("hasMedium",)
    hasMedium: Medium = Water
    _class_iri = S223.Connection


class WaterConnectionPoint(ConnectionPoint):
    _volatile = ("hasMedium",)
    hasMedium: Medium = Water


class WaterInletConnectionPoint(WaterConnectionPoint, InletConnectionPoint):
    _class_iri = S223.InletConnectionPoint


class WaterOutletConnectionPoint(WaterConnectionPoint, OutletConnectionPoint):
    _class_iri = S223.OutletConnectionPoint


class WaterBidirectionalConnectionPoint(
    WaterConnectionPoint, BidirectionalConnectionPoint
):
    _class_iri = S223.BidirectionalConnectionPoint


# === HOT WATER


class HotWaterConnection(Connection):
    hasMedium: Medium = Water.HotWater
    _class_iri = S223.Connection


class HotWaterConnectionPoint(ConnectionPoint):
    hasMedium: Medium = Water.HotWater


class HotWaterInletConnectionPoint(HotWaterConnectionPoint, InletConnectionPoint):
    _class_iri = S223.InletConnectionPoint


class HotWaterOutletConnectionPoint(HotWaterConnectionPoint, OutletConnectionPoint):
    _class_iri = S223.OutletConnectionPoint


# === MIXED WATER


class MixedWaterConnection(Connection):
    hasMedium: Medium = Water.MixedWater
    _class_iri = S223.Connection


class MixedWaterConnectionPoint(ConnectionPoint):
    hasMedium: Medium = Water.MixedWater


class MixedWaterInletConnectionPoint(InletConnectionPoint, MixedWaterConnectionPoint):
    _class_iri = S223.InletConnectionPoint


class MixedWaterOutletConnectionPoint(OutletConnectionPoint, MixedWaterConnectionPoint):
    _class_iri = S223.OutletConnectionPoint


# === STEAM


class SteamConnection(Connection):
    hasMedium: Medium = Water.Steam
    _class_iri = S223.Connection


class SteamConnectionPoint(ConnectionPoint):
    hasMedium: Medium = Water.Steam


class SteamInletConnectionPoint(InletConnectionPoint, SteamConnectionPoint):
    _class_iri = S223.InletConnectionPoint


class SteamOutletConnectionPoint(OutletConnectionPoint, SteamConnectionPoint):
    _class_iri = S223.OutletConnectionPoint


# === CHILLED WATER


class ChilledWaterConnection(Connection):
    hasMedium: Medium = Water.ChilledWater
    _class_iri = S223.Connection


class ChilledWaterConnectionPoint(ConnectionPoint):
    hasMedium: Medium = Water.ChilledWater


class ChilledWaterInletConnectionPoint(
    InletConnectionPoint, ChilledWaterConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class ChilledWaterOutletConnectionPoint(
    OutletConnectionPoint, ChilledWaterConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === Condensate


class CondensateConnection(Connection):
    hasMedium: Medium = Water.Condensate
    _class_iri = S223.Connection


class CondensateConnectionPoint(ConnectionPoint):
    hasMedium: Medium = Water.Condensate


class CondensateInletConnectionPoint(InletConnectionPoint, CondensateConnectionPoint):
    _class_iri = S223.InletConnectionPoint


class CondensateOutletConnectionPoint(OutletConnectionPoint, CondensateConnectionPoint):
    _class_iri = S223.OutletConnectionPoint


# === Condenser


class CondenserConnection(Connection):
    hasMedium: Medium = Water.Condenser
    _class_iri = S223.Connection


class CondenserConnectionPoint(ConnectionPoint):
    hasMedium: Medium = Water.Condenser


class CondenserInletConnectionPoint(InletConnectionPoint, CondenserConnectionPoint):
    _class_iri = S223.InletConnectionPoint


class CondenserOutletConnectionPoint(OutletConnectionPoint, CondenserConnectionPoint):
    _class_iri = S223.OutletConnectionPoint


# === Glycol Solution 15%


class Glycol15PercentConnection(Connection):
    hasMedium: Medium = GlycolSolution_15Percent
    _class_iri = S223.Connection


class Glycol15PercentConnectionPoint(ConnectionPoint):
    hasMedium: Medium = GlycolSolution_15Percent


class Glycol15PercentInletConnectionPoint(
    InletConnectionPoint, Glycol15PercentConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Glycol15PercentOutletConnectionPoint(
    OutletConnectionPoint, Glycol15PercentConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint


# === Glycol Solution 15%


class Glycol30PercentConnection(Connection):
    hasMedium: Medium = GlycolSolution_30Percent
    _class_iri = S223.Connection


class Glycol30PercentConnectionPoint(ConnectionPoint):
    hasMedium: Medium = GlycolSolution_30Percent


class Glycol30PercentInletConnectionPoint(
    InletConnectionPoint, Glycol15PercentConnectionPoint
):
    _class_iri = S223.InletConnectionPoint


class Glycol30PercentOutletConnectionPoint(
    OutletConnectionPoint, Glycol15PercentConnectionPoint
):
    _class_iri = S223.OutletConnectionPoint



from typing import Any


from bob.producer.causality import Causality
from bob.properties.ratio import Percent
from bob.properties.states import OnOffStatus

from ..core import (
    S223,
    PropertyReference,
)
from ..enum import Light
from ..properties import Count, Motion
from .sensor import Sensor, split_kwargs

_namespace = S223


class OccupancySensor(Sensor):
    _class_iri = S223.Sensor


class OccupantMotionSensor(OccupancySensor):
    _class_iri = S223.OccupantMotionSensor
    # measuresMedium: Medium = Light
    observes: PropertyReference  # Movement

    def __init__(self, **kwargs: Any) -> None:
        _sensor_kwargs, _property_kwargs = split_kwargs(kwargs)

        super().__init__(**_sensor_kwargs)

        self.observes = Motion(
            # isObservedBy=self,
            label=f"{self.label}.OccupantMotion",
            ofMedium=Light.Infrared,
            **_property_kwargs,
        )


# That should provide a producer of count value
class OccupantCounterSensor(OccupantMotionSensor):
    _class_iri = S223.Sensor
    # measuresMedium: Medium = Light
    occupantCount = Count

    def __init__(self, **kwargs: Any) -> None:
        _sensor_kwargs, _property_kwargs = split_kwargs(kwargs)

        super().__init__(**_sensor_kwargs)
        self.OccupantCount = Count(
            label=f"{self.label}.OccupantCount",
            **_property_kwargs,
        )
        counter = Causality(label="countProducer")
        counter.cause_input << self.observedProperty
        counter.effect_output >> self.occupantCount
        self > counter


# TODO : NOPE.... should observe something and produce a presence property
class OccupantPresenceSensor(OccupancySensor):
    _class_iri = S223.OccupantPresenceSensor
    # measuresMedium: Medium = Light
    observes: PropertyReference  # Intrusion...good for Windows and doors

    def __init__(self, **kwargs: Any) -> None:
        _sensor_kwargs, _measure_kwargs = split_kwargs(kwargs)

        super().__init__(**_sensor_kwargs)
        self.observes = OnOffStatus(
            # isObservedBy=self,
            label=f"{self.label}.OccupantPresence",
            **_measure_kwargs,
        )


class PositionSensor(Sensor):
    _class_iri = S223.Sensor
    observes: PropertyReference  # Movement

    def __init__(self, **kwargs: Any) -> None:
        _sensor_kwargs, _property_kwargs = split_kwargs(kwargs)

        super().__init__(**_sensor_kwargs)

        self.observes = Percent(
            # isObservedBy=self,
            label=f"{self.label}.Position",
            **_property_kwargs,
        )



from typing import Dict

from rdflib import URIRef

from bob.properties.electricity import ElectricPowerkW
from bob.properties.states import NormalAlarmStatus, OnOffCommand, OnOffStatus

from ...connections.air import CompressedAirOutletConnectionPoint
from ...connections.electricity import (
    Electricity_600VLL_3Ph_60HzInletConnectionPoint,
)
from ...connections.refrigerant import (
    RefrigerantInletConnectionPoint,
    RefrigerantOutletConnectionPoint,
)
from ...core import BOB, S223, Equipment, PropertyReference
from ...enum import (  # , R134a, R404a, R407c, R448a, R449a, R452a, R454b, R507a
    Refrigerant,
)

_namespace = BOB

compressor_template = {
    "cp": {"electricalInlet": Electricity_600VLL_3Ph_60HzInletConnectionPoint},
    "properties": {
        ("kW", ElectricPowerkW): {},
    },
}


class AirCompressor(Equipment):
    _class_iri: URIRef = S223.Compressor
    compressedAirOutlet: CompressedAirOutletConnectionPoint

    onOffStatus: OnOffStatus
    alarmStatus: NormalAlarmStatus
    onOffCommand: OnOffCommand

    # This will come from a sensor, but accessible from here
    outputPressure: PropertyReference

    def __init__(self, config: Dict = compressor_template, **kwargs):
        config["properties"] = config.get(
            "properties", compressor_template["properties"]
        )
        kwargs = {**config.get("params", {}), **kwargs}
        super().__init__(config, **kwargs)


class RefrigerationGasCompressor(Equipment):
    _class_iri: URIRef = S223.Compressor
    returnPort: RefrigerantInletConnectionPoint
    dischargePort: RefrigerantOutletConnectionPoint

    onOffStatus: OnOffStatus
    alarmStatus: NormalAlarmStatus
    onOffCommand: OnOffCommand

    # This will come from a sensor, but accessible from here
    dischargePressure: PropertyReference
    succionPressure: PropertyReference

    def __init__(self, config: Dict = compressor_template, **kwargs):
        config["properties"] = config.get(
            "properties", compressor_template["properties"]
        )
        kwargs = {**config.get("params", {}), **kwargs}
        super().__init__(config, **kwargs)
        self.dischargePort.paired_to(self.returnPort)

    def set_gas_type(self, gas: Refrigerant):
        self.set_medium(["returnPort", "dischargePort"], gas)



"""
This module contains just enough of the BACnet object model to help build
example models.  A complete description of BACnet objects and properties is
beyond the scope of this project.
"""

from __future__ import annotations

import logging
from typing import List

from rdflib import XSD, Literal, URIRef

from .core import (
    INCLUDE_INVERSE,
    ConnectionPoint,
    Node,
    bind_namespace,
    prefixes,
)
from .equipment.control.controller import Controller
from .externalreference.bacnet import BACnetExternalReference
from .multimethods import multimethod

# logging
_log = logging.getLogger(__name__)

# namespace
BACNET = bind_namespace("bacnet", prefixes["bacnet"])
_namespace = BACNET


class Device(Controller):
    _class_iri: URIRef = BACNET.Device
    _namespace = BACNET
    _device_object: DeviceObject  # reference to a device's DeviceObject
    _bacnet_objects = set()  # references to all objects

    deviceInstance: Literal


class Object(Node):
    _attr_uriref = {
        "objectIdentifier": BACNET["object-identifier"],
        "objectName": BACNET["object-name"],
        "objectType": BACNET["object-type"],
        "description": BACNET["description"],
    }
    _device: Device

    objectIdentifier: Literal
    objectName: Literal
    objectType: URIRef
    description: Literal

    _device: Device  # reference from an object to its device
    _present_value: BACnetExternalReference

    @property
    def presentValue(self) -> BACnetExternalReference:
        """
        Creates the present-value reference on demand to be used for a property.
        Cache it in case there are multiple references.
        """
        if getattr(self, "_present_value", None) is None:
            self._present_value = BACnetExternalReference(
                f"bacnet://{self._device.deviceInstance}/{self.objectIdentifier}/present-value"
            )
            self._data_graph.add(
                (self._node_iri, BACNET.hasProperty, self._present_value._node_iri)
            )
        return self._present_value

    @property
    def relinquishDefault(self) -> BACnetExternalReference:
        """
        Creates the relinquish-default reference on demand to be used for a property.
        Cache it in case there are multiple references.
        """
        if getattr(self, "_relinquish_default", None) is None:
            self._relinquish_default = BACnetExternalReference(
                f"bacnet://{self._device.deviceInstance}/{self.objectIdentifier}/relinquish-default"
            )
            self._data_graph.add(
                (self._node_iri, BACNET.hasProperty, self._relinquish_default._node_iri)
            )
        return self._relinquish_default


@multimethod
def contains_mm(device_: Device, object_: Object) -> None:
    """Device > Object"""
    _log.info(f"device {device_} contains object {object_}")

    # link the object to the device and vice versa
    object_._device = device_
    device_._bacnet_objects.add(object_)

    # the device instance number should match the object instance of the
    # device object
    if isinstance(object_, DeviceObject):
        if getattr(device_, "_device_object", None) is not None:
            raise ValueError(f"device already has a Device Object: {device_}")
        device_._device_object = object_

        device_instance = int(object_.objectIdentifier.split(",")[1])
        if device_.deviceInstance is not None:
            if device_.deviceInstance != device_instance:
                raise ValueError(f"device instance mismatch: {device_}")
        else:
            device_.deviceInstance = device_instance

    # add it to the graph
    device_._data_graph.add((device_._node_iri, BACNET.hasObject, object_._node_iri))

    if INCLUDE_INVERSE:
        device_._data_graph.add(
            (object_._node_iri, BACNET.isObjectOf, device_._node_iri)
        )


@multimethod
def contains_mm(device_: Device, object_list: List[Object]) -> None:
    """Device > [ Object, ... ]"""
    _log.info(f"device {device_} contains object list {object_list}")

    for object_ in object_list:
        contains_mm(device_, object_)


@multimethod
def connect_mm(object_: Object, cp_: ConnectionPoint) -> None:
    """Object >> CP"""
    _log.info(f"object {object_} mapsTo {cp_}")

    # add it to the graph
    object_._data_graph.add((object_._node_iri, BACNET.mapsTo, cp_._node_iri))

    # if INCLUDE_INVERSE:
    #    object_._data_graph.add(
    #        (object_._node_iri, BACNET.isObjectOf, device_._node_iri)
    #    )


@multimethod
def connect_mm(object1_: Object, object2_: Object) -> None:
    """Object >> Object"""
    _log.info(f"object {object1_} mapsTo {object2_}")

    # add it to the graph
    object1_._data_graph.add(
        (object1_._node_iri, BACNET.PeerToPeer, object2_._node_iri)
    )

    # if INCLUDE_INVERSE:
    #    object_._data_graph.add(
    #        (object_._node_iri, BACNET.isObjectOf, device_._node_iri)
    #    )


class DeviceObject(Object):
    objectType: URIRef = BACNET["ObjectType.device"]
    _attr_uriref = {
        "systemStatus": BACNET["system-status"],
        "vendorName": BACNET["vendor-name"],
        "vendorIdentifier": BACNET["vendor-identifier"],
        "modelName": BACNET["model-name"],
    }
    systemStatus: URIRef  # one of bacnet:DeviceStatus
    vendorName: Literal
    vendorIdentifier: XSD.nonNegativeInteger
    modelName: Literal


class AnalogInputObject(Object):
    objectType: URIRef = BACNET["ObjectType.analog-input"]


class AnalogOutputObject(Object):
    objectType: URIRef = BACNET["ObjectType.analog-output"]


class AnalogValueObject(Object):
    objectType: URIRef = BACNET["ObjectType.analog-value"]


class BinaryInputObject(Object):
    objectType: URIRef = BACNET["ObjectType.binary-input"]


class BinaryOutputObject(Object):
    objectType: URIRef = BACNET["ObjectType.binary-output"]


class BinaryValueObject(Object):
    objectType: URIRef = BACNET["ObjectType.binary-value"]


class CalendarObject(Object):
    objectType: URIRef = BACNET["ObjectType.calendar"]


class ScheduleObject(Object):
    objectType: URIRef = BACNET["ObjectType.schedule"]



from ..core import BOB, QUANTITYKIND, QuantifiableObservableProperty

_namespace = BOB

# all = [HP, Pressure, DifferentialStaticPressure]


class Length(QuantifiableObservableProperty):
    hasQuantityKind = QUANTITYKIND.Length


class Area(QuantifiableObservableProperty):
    hasQuantityKind = QUANTITYKIND.Area
